﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.AspNetCore.Identity;
using Benday.AspNetCore.SecuritySample.WebUi.Data;
using Benday.AspNetCore.SecuritySample.WebUi.Models;
using Benday.AspNetCore.SecuritySample.WebUi.Models.SecurityAdminViewModels;
using System.Security.Claims;

namespace Benday.AspNetCore.SecuritySample.WebUi.Controllers
{
    public class SecurityAdminController : Controller
    {
        private UserManager<ApplicationUser> _UserManager;
        private ApplicationDbContext _Context;
        private RoleManager<IdentityRole> _RoleManager;

        public SecurityAdminController(UserManager<ApplicationUser> userManager, RoleManager<IdentityRole> roleManager, ApplicationDbContext context)
        {
            if (roleManager == null)
            {
                throw new ArgumentNullException(nameof(roleManager), $"{nameof(roleManager)} is null.");
            }

            if (context == null)
            {
                throw new ArgumentNullException(nameof(context), $"{nameof(context)} is null.");
            }

            if (userManager == null)
            {
                throw new ArgumentNullException(nameof(userManager), $"{nameof(userManager)} is null.");
            }

            _UserManager = userManager;
            _Context = context;
            _RoleManager = roleManager;
        }

        public IActionResult Index()
        {
            var model = new SecurityAdminAction();

            model.LibraryEntries =
                _Context.LibraryEntries.ToList();

            model.Roles = AdaptToSelectListItems(_RoleManager.Roles.ToList());

            model.Users = AdaptToSelectListItems(_UserManager.Users.ToList());

            model.Permissions = new List<string>();
            model.Permissions.Add(SecurityConstants.PermissionName_View);
            model.Permissions.Add(SecurityConstants.PermissionName_Edit);

            return View(model);
        }

        [HttpPost]
        public async Task<ActionResult> AddToGroup(SecurityAdminAction securityAdminAction)
        {
            if (securityAdminAction != null &&
                securityAdminAction.RoleId != null &&
                securityAdminAction.UserId != null)
            {
                var role = await _RoleManager.FindByIdAsync(securityAdminAction.RoleId);
                var user = await _UserManager.FindByIdAsync(securityAdminAction.UserId);

                await _UserManager.AddToRoleAsync(user, role.Name);
            }

            return RedirectToAction("Index");
        }

        [HttpPost]
        public async Task<ActionResult> RemoveFromGroup(SecurityAdminAction securityAdminAction)
        {
            if (securityAdminAction != null &&
                securityAdminAction.RoleId != null &&
                securityAdminAction.UserId != null)
            {
                var role = await _RoleManager.FindByIdAsync(securityAdminAction.RoleId);
                var user = await _UserManager.FindByIdAsync(securityAdminAction.UserId);

                await _UserManager.RemoveFromRoleAsync(user, role.Name);
            }

            return RedirectToAction("Index");
        }
        
        [HttpPost]
        public async Task<ActionResult> SetAwesomeness(SecurityAdminAction securityAdminAction)
        {
            if (securityAdminAction != null &&
                securityAdminAction.AwesomenessLevel != null &&
                securityAdminAction.UserId != null)
            {
                var user = await _UserManager.FindByIdAsync(securityAdminAction.UserId);

                var claim = new Claim(SecurityConstants.Claim_LevelOfAwesomeness, 
                    securityAdminAction.AwesomenessLevel);

                var claims = await _UserManager.GetClaimsAsync(user);

                var removeThisClaim = (
                    from temp in claims
                    where temp.Type == SecurityConstants.Claim_LevelOfAwesomeness
                    select temp
                    ).FirstOrDefault();

                if (removeThisClaim != null)
                {
                    await _UserManager.RemoveClaimAsync(user, removeThisClaim);
                }

                await _UserManager.AddClaimAsync(user, claim);
            }

            return RedirectToAction("Index");
        }


        private List<SelectListItem> AdaptToSelectListItems(List<IdentityRole> fromValues)
        {
            List<SelectListItem> toValues = new List<SelectListItem>();

            foreach (var fromValue in fromValues)
            {
                toValues.Add(new SelectListItem()
                {
                    Value = fromValue.Id,
                    Text = fromValue.Name
                });
            }

            return toValues;
        }

        private List<SelectListItem> AdaptToSelectListItems(List<ApplicationUser> fromValues)
        {
            List<SelectListItem> toValues = new List<SelectListItem>();

            foreach (var fromValue in fromValues)
            {
                toValues.Add(new SelectListItem()
                {
                    Value = fromValue.Id,
                    Text = fromValue.UserName
                });
            }

            return toValues;
        }
    }
}