﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Benday.AspNetCore.SecuritySample.WebUi.Models;
using Microsoft.AspNetCore.Identity;
using Benday.AspNetCore.SecuritySample.WebUi.Data;
using Microsoft.EntityFrameworkCore;
using System.Security.Claims;

namespace Benday.AspNetCore.SecuritySample.WebUi.Controllers
{
    public class TestDataController : Controller
    {
        private UserManager<ApplicationUser> _UserManager;
        private ApplicationDbContext _Context;
        private RoleManager<IdentityRole> _RoleManager;

        public TestDataController(
            UserManager<ApplicationUser> userManager, 
            RoleManager<IdentityRole> roleManager,  
            ApplicationDbContext context)
        {
            if (roleManager == null)
            {
                throw new ArgumentNullException(nameof(roleManager), $"{nameof(roleManager)} is null.");
            }

            if (context == null)
            {
                throw new ArgumentNullException(nameof(context), $"{nameof(context)} is null.");
            }

            if (userManager == null)
            {
                throw new ArgumentNullException(nameof(userManager), $"{nameof(userManager)} is null.");
            }

            _UserManager = userManager;
            _Context = context;
            _RoleManager = roleManager;
        }

        public IActionResult Index()
        {
            List<ApplicationUser> users = _UserManager.Users.ToList();

            return View(users);
        }

        public async Task<IActionResult> CreateTestData()
        {
            await DeleteAllLibraryEntries();
            await DeleteAllRoles();
            await DeleteAllUsers();

            await CreateLibraryEntries();

            await CreateRolesAndGrantPermissions();
            
            await CreateUsersAndGrantPermissions();

            return RedirectToAction("Index");
        }

        private async Task CreateUsersAndGrantPermissions()
        {
            // create users
            var admin = await CreateUser(SecurityConstants.Username_Admin);
            var user1 = await CreateUser(SecurityConstants.Username_User1);
            var user2 = await CreateUser(SecurityConstants.Username_User2);
            var pluralsightEditor = await CreateUser(SecurityConstants.Username_Pluralsight);
            var bookEditor = await CreateUser(SecurityConstants.Username_Books);

            // add to roles
            await _UserManager.AddToRoleAsync(admin, SecurityConstants.RoleName_Admin);
            await _UserManager.AddToRoleAsync(pluralsightEditor, SecurityConstants.RoleName_PluralsightEditor);
            await _UserManager.AddToRoleAsync(bookEditor, SecurityConstants.RoleName_BookEditor);

            // grant permissions directly on library entries
            var allLibraryEntries = _Context.LibraryEntries.ToList();

            await _UserManager.AddToRoleAsync(user1, SecurityConstants.RoleName_User);
            await _UserManager.AddToRoleAsync(user2, SecurityConstants.RoleName_User);

            await AddPermissionToUser(_UserManager,
                user1,
                SecurityConstants.PermissionName_View,
                allLibraryEntries[0].Id);

            await AddPermissionToUser(_UserManager,
                user1,
                SecurityConstants.PermissionName_Edit,
                allLibraryEntries[0].Id);

            await AddPermissionToUser(_UserManager,
                user1,
                SecurityConstants.PermissionName_View,
                allLibraryEntries[1].Id);

            await AddPermissionToUser(_UserManager,
                user1,
                SecurityConstants.PermissionName_View,
                allLibraryEntries[2].Id);
        }

        private async Task DeleteAllRoles()
        {
            var roles = _RoleManager.Roles.ToList();

            foreach (var deleteThis in roles)
            {
                await _RoleManager.DeleteAsync(deleteThis);
            }
        }

        private async Task CreateRolesAndGrantPermissions()
        {
            // create the roles
            await _RoleManager.CreateAsync(new IdentityRole(SecurityConstants.RoleName_Admin));
            await _RoleManager.CreateAsync(new IdentityRole(SecurityConstants.RoleName_User));
            await _RoleManager.CreateAsync(new IdentityRole(SecurityConstants.RoleName_PluralsightEditor));
            await _RoleManager.CreateAsync(new IdentityRole(SecurityConstants.RoleName_BookEditor));

            // load the role objects
            var roleBookEditor = await _RoleManager.FindByNameAsync(SecurityConstants.RoleName_BookEditor);
            var rolePluralsightEditor = await _RoleManager.FindByNameAsync(SecurityConstants.RoleName_PluralsightEditor);

            // load the book and video library entries
            var books = _Context.LibraryEntries.Where(x => x.MediaType == SecurityConstants.MediaType_Book).ToList();            
            var videos = _Context.LibraryEntries.Where(x => x.MediaType == SecurityConstants.MediaType_Video).ToList();

            // grant permissions to the roles 
            foreach (var item in books)
            {
                await AddRoleClaim(_RoleManager, roleBookEditor, SecurityConstants.PermissionName_View, item);
                await AddRoleClaim(_RoleManager, roleBookEditor, SecurityConstants.PermissionName_Edit, item);
            }

            foreach (var item in videos)
            {
                await AddRoleClaim(_RoleManager, rolePluralsightEditor, SecurityConstants.PermissionName_View, item);
                await AddRoleClaim(_RoleManager, rolePluralsightEditor, SecurityConstants.PermissionName_Edit, item);
            }
        }

        private async Task AddRoleClaim(RoleManager<IdentityRole> roleManager, 
            IdentityRole toRole, 
            string permissionName, 
            LibraryEntry onLibraryEntry)
        {
            var claim = new Claim(permissionName, onLibraryEntry.Id.ToString());

            await roleManager.AddClaimAsync(toRole, claim);
        }

        private async Task<ApplicationUser> CreateUser(string username)
        {
            var user = new ApplicationUser();

            user.UserName = username;
            user.Email = username;
            user.EmailConfirmed = true;

            var result = await _UserManager.CreateAsync(user, "Password");

            if (result.Succeeded == false)
            {
                throw new InvalidOperationException("Error while creating user." + Environment.NewLine + result.Errors.ToString());
            }
            else
            {
                Console.WriteLine();
            }

            return user;
        }

        private async Task DeleteAllUsers()
        {
            var users = _UserManager.Users.ToList();

            foreach (var deleteThisUser in users)
            {
                await _UserManager.DeleteAsync(deleteThisUser);
            }
        }

        private async Task DeleteAllLibraryEntries()
        {
            var deleteThese = _Context.LibraryEntries.ToArray();

            _Context.LibraryEntries.RemoveRange(deleteThese);

            await _Context.SaveChangesAsync();
        }

        private async Task CreateLibraryEntries()
        {
            AddLibraryEntry(_Context.LibraryEntries,
                "DevOps Skills for Developers with TFS2017",
                new DateTime(2017, 10, 1),
                "Benjamin Day",
                "PLURALSIGHT.DEVOPS2017", SecurityConstants.MediaType_Video, 
                "https://www.pluralsight.com/courses/devops-skills-developers-visual-studio-tfs-2017");

            AddLibraryEntry(_Context.LibraryEntries,
                "Scrum Master Skills",
                new DateTime(2015, 8, 6),
                "Benjamin Day",
                "PLURALSIGHT.SMS", SecurityConstants.MediaType_Video,
                "http://app.pluralsight.com/courses/scrum-master-skills");

            AddLibraryEntry(_Context.LibraryEntries,
                "Real World Scrum with TFS2013",
                new DateTime(2014, 11, 10),
                "Benjamin Day",
                "PLURALSIGHT.RWS", SecurityConstants.MediaType_Video,
                "http://app.pluralsight.com/courses/real-world-scrum-team-foundation-server-2013");

            AddLibraryEntry(_Context.LibraryEntries,
                "Load Testing with VS2013",
                new DateTime(2014, 12, 5),
                "Benjamin Day",
                "PLURALSIGHT.LT2013", SecurityConstants.MediaType_Video,
                "http://app.pluralsight.com/courses/load-testing-visual-studio-2013");

            AddLibraryEntry(_Context.LibraryEntries,
                "Angular 2: Getting Started",
                new DateTime(2016, 4, 1),
                "Deborah Kurata",
                "PLURALSIGHT.ANG2", SecurityConstants.MediaType_Video,
                "http://app.pluralsight.com/courses/angular-2-getting-started");

            AddLibraryEntry(_Context.LibraryEntries,
                "C# Best Practices: Improving on the Basics",
                new DateTime(2015, 10, 29),
                "Deborah Kurata",
                "PLURALSIGHT.CSBP", SecurityConstants.MediaType_Video,
                "http://app.pluralsight.com/courses/csharp-best-practices-improving-basics");

            AddLibraryEntry(_Context.LibraryEntries,
                "The Wright Brothers",
                new DateTime(2015, 5, 5),
                "David McCullough",
                "DMCWBROS", SecurityConstants.MediaType_Book,
                "PLANES!");

            AddLibraryEntry(_Context.LibraryEntries,
                "The Bully Pulpit",
                new DateTime(2013, 11, 5),
                "Doris Kearns Goodwin",
                "DKGTBP", SecurityConstants.MediaType_Book,
                "Roosevelt, Taft, and journalism");

            AddLibraryEntry(_Context.LibraryEntries,
                "The Phoenix Project",
                new DateTime(2013, 1, 10),
                "Gene Kim",
                "GKTPP", SecurityConstants.MediaType_Book,
                "A story about DevOps.");

            await _Context.SaveChangesAsync();
        }

        private void AddLibraryEntry(DbSet<LibraryEntry> libraryEntries,
            string title,
            DateTime publishDate,
            string author,
            string catalogNumber,
            string mediaType,
            string summary)
        {
            var temp = new LibraryEntry();

            temp.Title = title;
            temp.PublishDate = publishDate;
            temp.Author = author;
            temp.CatalogNumber = catalogNumber;
            temp.MediaType = mediaType;
            temp.Summary = summary;

            libraryEntries.Add(temp);
        }

        private async Task AddPermissionToUser(UserManager<ApplicationUser> userManager, 
            ApplicationUser user, 
            string permissionName, 
            int itemId)
        {
            var claim = new System.Security.Claims.Claim(permissionName, itemId.ToString());

            await userManager.AddClaimAsync(user, claim);
        }
    }
}