﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Benday.AspNetCore.SecuritySample.WebUi.Data;
using Benday.AspNetCore.SecuritySample.WebUi.Models;
using Benday.AspNetCore.SecuritySample.WebUi.Services;
using Benday.AspNetCore.SecuritySample.WebUi.Security;
using Microsoft.AspNetCore.Authorization;

namespace Benday.AspNetCore.SecuritySample.WebUi
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddDbContext<ApplicationDbContext>(options =>
                options.UseSqlServer(Configuration.GetConnectionString("DefaultConnection")));

            services.AddIdentity<ApplicationUser, IdentityRole>(
                    options =>
                    {
                        options.Password.RequiredLength = 2;
                        options.Password.RequireNonAlphanumeric = false;
                        options.Password.RequireLowercase = false;
                        options.Password.RequireUppercase = false;
                        options.Password.RequireDigit = false;
                        options.Password.RequiredUniqueChars = 0;
                    }
                )
                .AddEntityFrameworkStores<ApplicationDbContext>()
                .AddDefaultTokenProviders();

            // Add application services.
            services.AddTransient<IEmailSender, EmailSender>();

            services.AddMvc();

            // setup authorization rules
            services.AddAuthorization(options =>
            {
                options.AddPolicy("AwesomenessPolicy",
                              policy => policy.Requirements.Add(
                                  new AwesomenessRequirement()
                                  {
                                      MinimumLevelOfAwesomeness = 5
                                  }
                                  ));

                options.AddPolicy("EditLibraryEntryPolicy",
                                  policy => policy.Requirements.Add(
                                      new EditLibraryEntryRequirement()));

                options.AddPolicy("ViewLibraryEntryPolicy",
                                  policy => policy.Requirements.Add(
                                      new LibraryEntryPermissionRequirement()
                                      {
                                          PermissionName = SecurityConstants.PermissionName_View
                                      }
                                      ));

                options.AddPolicy("DeleteLibraryEntryPolicy",
                                  policy => policy.Requirements.Add(
                                      new LibraryEntryPermissionRequirement()
                                      {
                                          PermissionName = "Library.Delete"
                                      }
                                      ));

            });


            // create instances
            services.AddSingleton<IAuthorizationHandler, EditLibraryEntryHandler>();
            services.AddSingleton<IAuthorizationHandler, LibraryEntryPermissionHandler>();
            services.AddSingleton<IAuthorizationHandler, AwesomenessHandler>();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                app.UseDatabaseErrorPage();
            }
            else
            {
                app.UseExceptionHandler("/Home/Error");
            }

            app.UseStaticFiles();

            app.UseAuthentication();

            app.UseMvc(routes =>
            {
                routes.MapRoute(
                    name: "default",
                    template: "{controller=Home}/{action=Index}/{id?}");
            });
        }
    }
}
